@@ -216,6 +216,8 @@ static int PylibMC_Client_init(PylibMC_Client *self, PyObject *args,
     }
     self->native_deserialization = (uint8_t) native_deserialization;
 
+    self->pickle_protocol = -1;
+
     while ((c_srv = PyIter_Next(srvs_it)) != NULL) {
         unsigned char stype;
         char *hostname;
@@ -352,9 +354,8 @@ static int _PylibMC_Deflate(char *value, Py_ssize_t value_len,
         goto error;
     }
 
-    if(strm.total_out >= value_len) {
-      /* if we didn't actually save anything, don't bother storing it
-         compressed */
+    if ((Py_ssize_t)strm.total_out >= value_len) {
+      /* if no data was saved, don't use compression */
       goto error;
     }
 
@@ -594,7 +595,7 @@ static PyObject *_PylibMC_parse_memcached_value(PylibMC_Client *self,
 #else
     if (flags & PYLIBMC_FLAG_ZLIB) {
         PyErr_SetString(PylibMCExc_Error,
-            "value for key compressed, unable to inflate");
+            "key is compressed but pylibmc is compiled without zlib support");
         return NULL;
     }
 #endif
@@ -645,20 +646,21 @@ static PyObject *_PylibMC_deserialize_native(PylibMC_Client *self, PyObject *val
 
     switch (dtype) {
         case PYLIBMC_FLAG_PICKLE:
-            retval = value ? _PylibMC_Unpickle_Bytes(value) : _PylibMC_Unpickle(value_str, value_size);
+            retval = value ? _PylibMC_Unpickle_Bytes(self, value) : _PylibMC_Unpickle(self, value_str, value_size);
             break;
         case PYLIBMC_FLAG_INTEGER:
         case PYLIBMC_FLAG_LONG:
-        case PYLIBMC_FLAG_BOOL:
             if (value) {
                 retval = PyLong_FromString(PyBytes_AS_STRING(value), NULL, 10);
             } else {
                 retval = _PyLong_FromStringAndSize(value_str, value_size, NULL, 10);;
             }
-            if (retval != NULL && dtype == PYLIBMC_FLAG_BOOL) {
-                PyObject *bool_retval = PyBool_FromLong(PyLong_AS_LONG(retval));
-                Py_DECREF(retval);
-                retval = bool_retval;
+            break;
+        case PYLIBMC_FLAG_TEXT:
+            if (value) {
+                retval = PyUnicode_FromEncodedObject(value, "utf-8", "strict");
+            } else {
+                retval = PyUnicode_FromStringAndSize(value_str, value_size);
             }
             break;
         case PYLIBMC_FLAG_NONE:
@@ -723,7 +725,7 @@ static PyObject *PylibMC_Client_get(PylibMC_Client *self, PyObject *args) {
 
     if (!_key_normalized_obj(&key)) {
         return NULL;
-    } else if (!PySequence_Length(key) ) {
+    } else if (!PySequence_Length(key)) {
         Py_INCREF(default_value);
         return default_value;
     }
@@ -736,18 +738,23 @@ static PyObject *PylibMC_Client_get(PylibMC_Client *self, PyObject *args) {
 
     Py_DECREF(key);
 
-    if (mc_val != NULL) {
+    if (error == MEMCACHED_SUCCESS) {
+        /* note that mc_val can and is NULL for zero-length values. */
         PyObject *r = _PylibMC_parse_memcached_value(self, mc_val, val_size, flags);
-        free(mc_val);
+
+        if (mc_val != NULL) {
+            free(mc_val);
+        }
+
         if (_PylibMC_cache_miss_simulated(r)) {
             Py_INCREF(default_value);
             return default_value;
         }
+
         return r;
-    } else if (error == MEMCACHED_SUCCESS) {
-        /* This happens for empty values, and so we fake an empty string. */
-        return PyBytes_FromStringAndSize("", 0);
-    } else if (error == MEMCACHED_NOTFOUND) {
+    }
+
+    if (error == MEMCACHED_NOTFOUND) {
         Py_INCREF(default_value);
         return default_value;
     }
@@ -1247,12 +1254,12 @@ static int _PylibMC_serialize_native(PylibMC_Client *self, PyObject *value_obj,
         /* Make store_val an owned reference */
         store_val = value_obj;
         Py_INCREF(store_val);
+    } else if (PyUnicode_Check(value_obj)) {
+        store_flags = PYLIBMC_FLAG_TEXT;
+        store_val = PyUnicode_AsUTF8String(value_obj);
     } else if (PyBool_Check(value_obj)) {
-        store_flags |= PYLIBMC_FLAG_BOOL;
-        /* bool cannot be subclassed; there are only two singleton values,
-           Py_True and Py_False */
-        const char *value_str = (value_obj == Py_True) ? "1" : "0";
-        store_val = PyBytes_FromString(value_str);
+        store_flags |= PYLIBMC_FLAG_INTEGER;
+        store_val = PyBytes_FromStringAndSize(&"01"[value_obj == Py_True], 1);
 #if PY_MAJOR_VERSION >= 3
     } else if (PyLong_Check(value_obj)) {
         store_flags |= PYLIBMC_FLAG_LONG;
@@ -1275,7 +1282,7 @@ static int _PylibMC_serialize_native(PylibMC_Client *self, PyObject *value_obj,
         /* we have no idea what it is, so we'll store it pickled */
         Py_INCREF(value_obj);
         store_flags |= PYLIBMC_FLAG_PICKLE;
-        store_val = _PylibMC_Pickle(value_obj);
+        store_val = _PylibMC_Pickle(self, value_obj);
         Py_DECREF(value_obj);
     }
 
@@ -2128,14 +2135,21 @@ static PyObject *PylibMC_Client_get_behaviors(PylibMC_Client *self) {
         uint64_t bval;
         PyObject *x;
 
-        bval = memcached_behavior_get(self->mc, b->flag);
+        switch (b->flag) {
+        case PYLIBMC_BEHAVIOR_PICKLE_PROTOCOL:
+            bval = self->pickle_protocol;
+            break;
+        default:
+            bval = memcached_behavior_get(self->mc, b->flag);
+        }
+
         x = PyLong_FromLong((long)bval);
         if (x == NULL || PyDict_SetItemString(retval, b->name, x) == -1) {
             Py_XDECREF(x);
             goto error;
         }
-
         Py_DECREF(x);
+
     }
 
     return retval;
@@ -2148,7 +2162,7 @@ static PyObject *PylibMC_Client_set_behaviors(PylibMC_Client *self,
         PyObject *behaviors) {
     PylibMC_Behavior *b;
     PyObject *py_v;
-    uint64_t v;
+    long v;
     memcached_return r;
     char *key;
 
@@ -2163,16 +2177,23 @@ static PyObject *PylibMC_Client_set_behaviors(PylibMC_Client *self,
             goto error;
         }
 
-        v = (uint64_t)PyLong_AS_LONG(py_v);
+        v = PyLong_AsLong(py_v);
         Py_DECREF(py_v);
 
-        r = memcached_behavior_set(self->mc, b->flag, v);
-        if (r != MEMCACHED_SUCCESS) {
-            PyErr_Format(PylibMCExc_Error,
-                         "memcached_behavior_set returned %d for "
-                         "behavior '%.32s' = %u", r, b->name, (unsigned int)v);
-            goto error;
+        switch (b->flag) {
+        case PYLIBMC_BEHAVIOR_PICKLE_PROTOCOL:
+            self->pickle_protocol = v;
+            break;
+        default:
+            r = memcached_behavior_set(self->mc, b->flag, (uint64_t)v);
+            if (r != MEMCACHED_SUCCESS) {
+                PyErr_Format(PylibMCExc_Error,
+                             "memcached_behavior_set returned %d for "
+                             "behavior '%.32s' = %ld", r, b->name, v);
+                goto error;
+            }
         }
+
     }
 
     for (b = PylibMC_callbacks; b->name != NULL; b++) {
@@ -2383,6 +2404,7 @@ static PyObject *PylibMC_Client_clone(PylibMC_Client *self) {
     Py_END_ALLOW_THREADS;
     clone->native_serialization = self->native_serialization;
     clone->native_deserialization = self->native_deserialization;
+    clone->pickle_protocol = self->pickle_protocol;
     return (PyObject *)clone;
 }
 /* }}} */
@@ -2471,7 +2493,7 @@ static PyObject *_PylibMC_GetPickles(const char *attname) {
     return pickle_attr;
 }
 
-static PyObject *_PylibMC_Unpickle(const char *buff, Py_ssize_t size) {
+static PyObject *_PylibMC_Unpickle(PylibMC_Client *self, const char *buff, Py_ssize_t size) {
 #if PY_MAJOR_VERSION >= 3
         return PyObject_CallFunction(_PylibMC_pickle_loads, "y#", buff, size);
 #else
@@ -2479,12 +2501,12 @@ static PyObject *_PylibMC_Unpickle(const char *buff, Py_ssize_t size) {
 #endif
 }
 
-static PyObject *_PylibMC_Unpickle_Bytes(PyObject *val) {
+static PyObject *_PylibMC_Unpickle_Bytes(PylibMC_Client *self, PyObject *val) {
     return PyObject_CallFunctionObjArgs(_PylibMC_pickle_loads, val, NULL);
 }
 
-static PyObject *_PylibMC_Pickle(PyObject *val) {
-    return PyObject_CallFunction(_PylibMC_pickle_dumps, "Oi", val, -1);
+static PyObject *_PylibMC_Pickle(PylibMC_Client *self, PyObject *val) {
+    return PyObject_CallFunction(_PylibMC_pickle_dumps, "Oi", val, self->pickle_protocol);
 }
 /* }}} */
 
