@@ -23,14 +23,6 @@ class PylibmcTestCase(unittest.TestCase):
         self.mc.disconnect_all()
         del self.mc
 
-def dump_infos():
-    if hasattr(_pylibmc, "__file__"):
-        print("Starting tests with _pylibmc at", _pylibmc.__file__)
-    else:
-        print("Starting tests with static _pylibmc:", _pylibmc)
-    print("Reported libmemcached version:", _pylibmc.libmemcached_version)
-    print("Reported pylibmc version:", _pylibmc.__version__)
-    print("Support compression:", _pylibmc.support_compression)
 
 def get_refcounts(refcountables):
     """Measure reference counts during testing.
